/**
 * Types supported by {@link IConfigProviderBase}.
 * @public
 */
export declare type ConfigTypes = string | number | boolean | number[] | string[] | boolean[] | undefined;

/**
 * Allows an interface to extend interfaces that already extend an {@link IEventProvider}.
 *
 * @example
 *
 * ``` typescript
 * interface AEvents extends IEvent{
 *  (event: "a-event",listener: (a: number)=>void);
 * }
 * interface A extends IEventProvider<AEvents>{
 *  a: object;
 * }
 *
 * interface BEvents extends IEvent{
 *  (event: "b-event",listener: (b: string)=>void);
 * }
 * interface B extends ExtendEventProvider<AEvents, A, BEvents>{
 *  b: boolean;
 * };
 * ```
 *
 * interface B will now extend interface A and its events
 * @public
 */
export declare type ExtendEventProvider<TBaseEvent extends IEvent, TBase extends IEventProvider<TBaseEvent>, TEvent extends TBaseEvent> = Omit<Omit<Omit<TBase, "on">, "once">, "off"> & IEventProvider<TBaseEvent> & IEventProvider<TEvent>;

/**
 * Error types the Fluid Framework may report.
 * @alpha
 */
export declare const FluidErrorTypes: {
    /**
     * Some error, most likely an exception caught by runtime and propagated to container as critical error
     */
    readonly genericError: "genericError";
    /**
     * Throttling error from server. Server is busy and is asking not to reconnect for some time
     */
    readonly throttlingError: "throttlingError";
    /**
     * Data loss error detected by Container / DeltaManager. Likely points to storage issue.
     */
    readonly dataCorruptionError: "dataCorruptionError";
    /**
     * Error encountered when processing an operation. May correlate with data corruption.
     */
    readonly dataProcessingError: "dataProcessingError";
    /**
     * Error indicating an API is being used improperly resulting in an invalid operation.
     */
    readonly usageError: "usageError";
};

/**
 * @alpha
 */
export declare type FluidErrorTypes = (typeof FluidErrorTypes)[keyof typeof FluidErrorTypes];

/**
 * This utility type take interface(s) that follow the FluidObject pattern, and produces
 * a new type that can be used for inspection and discovery of those interfaces.
 *
 * It is meant to be used with types that are known to implement the FluidObject pattern.
 * A common way to specify a type implements the FluidObject pattern is to expose it as a
 * FluidObject without a generic argument.
 *
 * @example
 *
 * For example, if we have an interface like the following:
 *
 * ```typescript
 * interface IProvideFoo{
 *  IFoo: IFoo
 * }
 * interface IFoo extends IProvideFoo{
 *  foobar();
 * }
 * ```
 *
 * and a function that returns a FluidObject. You would do the following
 *
 * `const maybeFoo: FluidObject<IFoo> = getFluidObject()`;
 *
 * Either IFoo or IProvideFoo are valid generic arguments. In both case
 * maybeFoo will be of type `{IFoo?: IFoo}`. If IFoo is not undefined,
 * then the FluidObject provides IFoo, and it can be used.
 *
 * You can inspect multiple types via a intersection. For example:
 * `FluidObject<IFoo & IBar>`
 * @public
 */
export declare type FluidObject<T = unknown> = {
    [P in FluidObjectProviderKeys<T>]?: T[P];
};

/**
 * This utility type creates a type that is the union of all keys on the generic type
 * which implement the FluidObject pattern.
 *
 * See {@link FluidObject}
 *
 * For example `FluidObjectKeys<IFoo & IBar>` would result in `"IFoo" | "IBar"`
 * @public
 */
export declare type FluidObjectKeys<T> = keyof FluidObject<T>;

/**
 * Produces a valid FluidObject key given a type and a property.
 *
 * @remarks
 *
 * A valid FluidObject key is a property that exists on the incoming type
 * as well as on the type of the property itself. For example: `IProvideFoo.IFoo.IFoo`
 * This aligns with the FluidObject pattern expected to be used with all FluidObjects.
 *
 * This utility type is meant for internal use by {@link FluidObject}
 *
 * @example
 *
 * ```typescript
 * interface IProvideFoo{
 *  IFoo: IFoo
 * }
 * interface IFoo extends IProvideFoo{
 *  foobar();
 * }
 * ```
 *
 * This pattern enables discovery, and delegation in a standard way which is central
 * to FluidObject pattern.
 * @public
 */
export declare type FluidObjectProviderKeys<T, TProp extends keyof T = keyof T> = string extends TProp ? never : number extends TProp ? never : TProp extends keyof Required<T>[TProp] ? Required<T>[TProp] extends Required<Required<T>[TProp]>[TProp] ? TProp : never : never;

/**
 * Base interface for providing configurations to enable/disable/control features.
 * @public
 */
export declare interface IConfigProviderBase {
    /**
     * For the specified config name this function gets the value.
     *
     * This type is meant be easy to implement by Fluid Framework consumers
     * so the returned valued needs minimal type coercion, and allows consumers to
     * return values in a natural way from whatever source they retrieve them.
     *
     * For instance a value of 1 maybe be returned as a string or a number.
     * For array types a json string or an object are allowable.
     *
     * It should return undefined if there is no value available for the config name.
     *
     * @param name - The name of the config to get the value for.
     *
     * @privateRemarks Generally, this type should only be taken as input, and be wrapped by an
     * internal monitoring context from the fluidframework/telemetry-utils package. That will provide
     * a wrapper with provides strongly typed access to values via consistent type coercion.
     */
    getRawConfig(name: string): ConfigTypes;
}

/**
 * Base interface for objects that require lifetime management via explicit disposal.
 * @public
 */
export declare interface IDisposable {
    /**
     * Whether or not the object has been disposed.
     * If true, the object should be considered invalid, and its other state should be disregarded.
     */
    readonly disposed: boolean;
    /**
     * Dispose of the object and its resources.
     * @param error - Optional error indicating the reason for the disposal, if the object was
     * disposed as the result of an error.
     */
    dispose(error?: Error): void;
}

/**
 * Base interface for all errors and warnings emitted the container.
 *
 * @remarks
 *
 * We are in the process of unifying error types across layers of the Framework. For now we have only migrated
 * those from container-definitions. Once fully migrated, this will be a base interface for all errors and
 * warnings emitted by the Fluid Framework. Currently only the container layer is using IErrorBase.
 * Runtime and others will follow soon.
 * @public
 */
export declare interface IErrorBase extends Partial<Error> {
    /**
     * A type tag differentiating kinds of errors emitted by the container.
     *
     * @see See {@link FluidErrorTypes#genericError} for some common examples.
     * - container
     * - runtime
     * - drivers
     */
    readonly errorType: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Error | Error.message}
     *
     * @remarks
     *
     * Privacy Note - This is a freeform string that we may not control in all cases (e.g. a dependency throws an error)
     * If there are known cases where this contains privacy-sensitive data it will be tagged and included in the result
     * of getTelemetryProperties. When logging, consider fetching it that way rather than straight from this field.
     */
    readonly message: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Error/name | Error.name}
     */
    readonly name?: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Error/stack | Error.stack}
     */
    readonly stack?: string;
    /**
     * Returns all properties of this error object that are fit for logging.
     * Some may be tagged to indicate they contain some kind of sensitive data.
     */
    getTelemetryProperties?(): ITelemetryBaseProperties;
}

/**
 * Base interface for error event emitters.
 * @public
 */
export declare interface IErrorEvent extends IEvent {
    /**
     * Base error event emitter signature.
     *
     * @eventProperty
     */
    (event: "error", listener: (message: any) => void): any;
}

/**
 * Base interface for event emitters.
 * @public
 */
export declare interface IEvent {
    /**
     * Base event emitter signature.
     *
     * @remarks The event emitter polyfill and the node event emitter have different event types:
     * `string | symbol` vs. `string | number`.
     *
     * So for our typing we'll contrain to string, that way we work with both.
     *
     * @eventProperty
     */
    (event: string, listener: (...args: any[]) => void): any;
}

/**
 * Base interface for event providers.
 * @public
 */
export declare interface IEventProvider<TEvent extends IEvent> {
    /**
     * Registers a callback to be invoked when the corresponding event is triggered.
     */
    readonly on: IEventTransformer<this, TEvent>;
    /**
     * Registers a callback to be invoked the first time (after registration) the corresponding event is triggered.
     */
    readonly once: IEventTransformer<this, TEvent>;
    /**
     * Removes the corresponding event if it has been registered.
     */
    readonly off: IEventTransformer<this, TEvent>;
}

/**
 * The placeholder type that should be used instead of `this` in events.
 * @public
 */
export declare type IEventThisPlaceHolder = {
    thisPlaceHolder: "thisPlaceHolder";
};

/**
 * This type is a conditional type for transforming all the overloads provided in `TEvent`.
 *
 * @remarks
 * Due to limitations of the TypeScript typing system, we need to handle each number of overload individually.
 * It currently supports the max of 15 event overloads which is more than we use anywhere.
 * At more than 15 overloads we start to hit {@link https://github.com/microsoft/TypeScript/issues/37209 | TS2589}.
 * If we need to move beyond 15 we should evaluate using a mapped type pattern like `{"event":(listenerArgs)=>void}`
 * @public
 */
export declare type IEventTransformer<TThis, TEvent extends IEvent> = TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: infer E6, listener: (...args: infer A6) => void): any;
    (event: infer E7, listener: (...args: infer A7) => void): any;
    (event: infer E8, listener: (...args: infer A8) => void): any;
    (event: infer E9, listener: (...args: infer A9) => void): any;
    (event: infer E10, listener: (...args: infer A10) => void): any;
    (event: infer E11, listener: (...args: infer A11) => void): any;
    (event: infer E12, listener: (...args: infer A12) => void): any;
    (event: infer E13, listener: (...args: infer A13) => void): any;
    (event: infer E14, listener: (...args: infer A14) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> & TransformedEvent<TThis, E6, A6> & TransformedEvent<TThis, E7, A7> & TransformedEvent<TThis, E8, A8> & TransformedEvent<TThis, E9, A9> & TransformedEvent<TThis, E10, A10> & TransformedEvent<TThis, E11, A11> & TransformedEvent<TThis, E12, A12> & TransformedEvent<TThis, E13, A13> & TransformedEvent<TThis, E14, A14> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: infer E6, listener: (...args: infer A6) => void): any;
    (event: infer E7, listener: (...args: infer A7) => void): any;
    (event: infer E8, listener: (...args: infer A8) => void): any;
    (event: infer E9, listener: (...args: infer A9) => void): any;
    (event: infer E10, listener: (...args: infer A10) => void): any;
    (event: infer E11, listener: (...args: infer A11) => void): any;
    (event: infer E12, listener: (...args: infer A12) => void): any;
    (event: infer E13, listener: (...args: infer A13) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> & TransformedEvent<TThis, E6, A6> & TransformedEvent<TThis, E7, A7> & TransformedEvent<TThis, E8, A8> & TransformedEvent<TThis, E9, A9> & TransformedEvent<TThis, E10, A10> & TransformedEvent<TThis, E11, A11> & TransformedEvent<TThis, E12, A12> & TransformedEvent<TThis, E13, A13> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: infer E6, listener: (...args: infer A6) => void): any;
    (event: infer E7, listener: (...args: infer A7) => void): any;
    (event: infer E8, listener: (...args: infer A8) => void): any;
    (event: infer E9, listener: (...args: infer A9) => void): any;
    (event: infer E10, listener: (...args: infer A10) => void): any;
    (event: infer E11, listener: (...args: infer A11) => void): any;
    (event: infer E12, listener: (...args: infer A12) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> & TransformedEvent<TThis, E6, A6> & TransformedEvent<TThis, E7, A7> & TransformedEvent<TThis, E8, A8> & TransformedEvent<TThis, E9, A9> & TransformedEvent<TThis, E10, A10> & TransformedEvent<TThis, E11, A11> & TransformedEvent<TThis, E12, A12> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: infer E6, listener: (...args: infer A6) => void): any;
    (event: infer E7, listener: (...args: infer A7) => void): any;
    (event: infer E8, listener: (...args: infer A8) => void): any;
    (event: infer E9, listener: (...args: infer A9) => void): any;
    (event: infer E10, listener: (...args: infer A10) => void): any;
    (event: infer E11, listener: (...args: infer A11) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> & TransformedEvent<TThis, E6, A6> & TransformedEvent<TThis, E7, A7> & TransformedEvent<TThis, E8, A8> & TransformedEvent<TThis, E9, A9> & TransformedEvent<TThis, E10, A10> & TransformedEvent<TThis, E11, A11> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: infer E6, listener: (...args: infer A6) => void): any;
    (event: infer E7, listener: (...args: infer A7) => void): any;
    (event: infer E8, listener: (...args: infer A8) => void): any;
    (event: infer E9, listener: (...args: infer A9) => void): any;
    (event: infer E10, listener: (...args: infer A10) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> & TransformedEvent<TThis, E6, A6> & TransformedEvent<TThis, E7, A7> & TransformedEvent<TThis, E8, A8> & TransformedEvent<TThis, E9, A9> & TransformedEvent<TThis, E10, A10> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: infer E6, listener: (...args: infer A6) => void): any;
    (event: infer E7, listener: (...args: infer A7) => void): any;
    (event: infer E8, listener: (...args: infer A8) => void): any;
    (event: infer E9, listener: (...args: infer A9) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> & TransformedEvent<TThis, E6, A6> & TransformedEvent<TThis, E7, A7> & TransformedEvent<TThis, E8, A8> & TransformedEvent<TThis, E9, A9> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: infer E6, listener: (...args: infer A6) => void): any;
    (event: infer E7, listener: (...args: infer A7) => void): any;
    (event: infer E8, listener: (...args: infer A8) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> & TransformedEvent<TThis, E6, A6> & TransformedEvent<TThis, E7, A7> & TransformedEvent<TThis, E8, A8> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: infer E6, listener: (...args: infer A6) => void): any;
    (event: infer E7, listener: (...args: infer A7) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> & TransformedEvent<TThis, E6, A6> & TransformedEvent<TThis, E7, A7> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: infer E6, listener: (...args: infer A6) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> & TransformedEvent<TThis, E6, A6> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: infer E5, listener: (...args: infer A5) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> & TransformedEvent<TThis, E5, A5> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: infer E4, listener: (...args: infer A4) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> & TransformedEvent<TThis, E4, A4> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: infer E3, listener: (...args: infer A3) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> & TransformedEvent<TThis, E3, A3> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: infer E2, listener: (...args: infer A2) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> & TransformedEvent<TThis, E2, A2> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: infer E1, listener: (...args: infer A1) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> & TransformedEvent<TThis, E1, A1> : TEvent extends {
    (event: infer E0, listener: (...args: infer A0) => void): any;
    (event: string, listener: (...args: any[]) => void): any;
} ? TransformedEvent<TThis, E0, A0> : TransformedEvent<TThis, string, any[]>;

/**
 * Data structure used to describe the code to load on the Fluid document.
 *
 * @deprecated in favor of {@link @fluidframework/container-definitions#IFluidCodeDetails}
 * to have code loading modules in same package.
 * @alpha
 */
export declare interface IFluidCodeDetails {
    /**
     * The code package to be used on the Fluid document. This is either the package name which will be loaded
     * from a package manager. Or the expanded Fluid package.
     */
    readonly package: string | Readonly<IFluidPackage>;
    /**
     * Configuration details. This includes links to the package manager and base CDNs.
     */
    readonly config?: IFluidCodeDetailsConfig;
}

/* Excluded from this release type: IFluidCodeDetailsComparer */

/**
 * Package manager configuration. Provides a key value mapping of config values.
 *
 * @deprecated in favor of {@link @fluidframework/container-definitions#IFluidCodeDetailsConfig}
 * to have code loading modules in same package.
 * @alpha
 */
export declare interface IFluidCodeDetailsConfig {
    readonly [key: string]: string;
}

/**
 * @public
 */
export declare const IFluidHandle: keyof IProvideFluidHandle;

/**
 * Handle to a shared {@link FluidObject}.
 * @public
 */
export declare interface IFluidHandle<T = FluidObject & IFluidLoadable> extends IProvideFluidHandle {
    /**
     * @deprecated Do not use handle's path for routing. Use `get` to get the underlying object.
     *
     * The absolute path to the handle context from the root.
     */
    readonly absolutePath: string;
    /**
     * Flag indicating whether or not the entity has services attached.
     */
    readonly isAttached: boolean;
    /**
     * @deprecated To be removed. This is part of an internal API surface and should not be called.
     *
     * Runs through the graph and attach the bounded handles.
     */
    attachGraph(): void;
    /**
     * Returns a promise to the Fluid Object referenced by the handle.
     */
    get(): Promise<T>;
    /**
     * @deprecated To be removed. This is part of an internal API surface and should not be called.
     *
     * Binds the given handle to this one or attach the given handle if this handle is attached.
     * A bound handle will also be attached once this handle is attached.
     */
    bind(handle: IFluidHandle): void;
}

/**
 * @public
 */
export declare const IFluidHandleContext: keyof IProvideFluidHandleContext;

/**
 * Describes a routing context from which other `IFluidHandleContext`s are defined.
 * @public
 */
export declare interface IFluidHandleContext extends IProvideFluidHandleContext {
    /**
     * The absolute path to the handle context from the root.
     */
    readonly absolutePath: string;
    /**
     * The parent IFluidHandleContext that has provided a route path to this IFluidHandleContext or undefined
     * at the root.
     */
    readonly routeContext?: IFluidHandleContext;
    /**
     * Flag indicating whether or not the entity has services attached.
     */
    readonly isAttached: boolean;
    /**
     * Runs through the graph and attach the bounded handles.
     */
    attachGraph(): void;
    resolveHandle(request: IRequest): Promise<IResponse>;
}

/**
 * @public
 */
export declare const IFluidLoadable: keyof IProvideFluidLoadable;

/**
 * A shared FluidObject has a URL from which it can be referenced
 * @public
 */
export declare interface IFluidLoadable extends IProvideFluidLoadable {
    handle: IFluidHandle;
}

/**
 * Fluid-specific properties expected on a package to be loaded by the code loader.
 * While compatible with the npm package format it is not necessary that that package is an
 * npm package:
 * {@link https://stackoverflow.com/questions/10065564/add-custom-metadata-or-config-to-package-json-is-it-valid}
 *
 * @deprecated In favor of {@link @fluidframework/container-definitions#IFluidPackage}
 * to have code loading modules in same package.
 * @alpha
 */
export declare interface IFluidPackage {
    /**
     * The name of the package that this code represnets
     */
    name: string;
    /**
     * This object represents the Fluid specific properties of the package
     */
    fluid: {
        /**
         * The name of the of the environment. This should be something like browser, or node
         * and contain the necessary targets for loading this code in that environment.
         */
        [environment: string]: undefined | IFluidPackageEnvironment;
    };
    /**
     * General access for extended fields as specific usages will
     * likely have additional infornamation like a definition of
     * compatible versions, or deployment information like rings or rollouts.
     */
    [key: string]: unknown;
}

/**
 * Specifies an environment on Fluid property of an {@link IFluidPackage}.
 *
 * @deprecated In favor of {@link @fluidframework/container-definitions#IFluidPackageEnvironment}
 * to have code loading modules in same package.
 * @alpha
 */
export declare interface IFluidPackageEnvironment {
    /**
     * The name of the target. For a browser environment, this could be umd for scripts
     * or css for styles.
     */
    [target: string]: undefined | {
        /**
         * List of files for the target. These can be relative or absolute.
         * The code loader should resolve relative paths, and validate all
         * full urls.
         */
        files: string[];
        /**
         * General access for extended fields as specific usages will
         * likely have additional infornamation like a definition
         * of Library, the entrypoint for umd packages
         */
        [key: string]: unknown;
    };
}

/* Excluded from this release type: IFluidRunnable */

/* Excluded from this release type: IGenericError */

/* Excluded from this release type: ILoggingError */

/* Excluded from this release type: IProvideFluidCodeDetailsComparer */

/**
 * @public
 */
export declare interface IProvideFluidHandle {
    readonly IFluidHandle: IFluidHandle;
}

/**
 * @public
 */
export declare interface IProvideFluidHandleContext {
    readonly IFluidHandleContext: IFluidHandleContext;
}

/**
 * @public
 */
export declare interface IProvideFluidLoadable {
    readonly IFluidLoadable: IFluidLoadable;
}

/* Excluded from this release type: IProvideFluidRunnable */

/**
 * @public
 */
export declare interface IRequest {
    url: string;
    headers?: IRequestHeader;
}

/**
 * @public
 */
export declare interface IRequestHeader {
    [index: string]: any;
}

/**
 * @public
 */
export declare interface IResponse {
    mimeType: string;
    status: number;
    value: any;
    headers?: {
        [key: string]: any;
    };
    stack?: string;
}

/* Excluded from this release type: isFluidCodeDetails */

/* Excluded from this release type: isFluidPackage */

/* Excluded from this release type: ISignalEnvelope */

/**
 * Base interface for logging telemetry statements.
 * Can contain any number of properties that get serialized as json payload.
 * @param category - category of the event, like "error", "performance", "generic", etc.
 * @param eventName - name of the event.
 * @public
 */
export declare interface ITelemetryBaseEvent extends ITelemetryBaseProperties {
    category: string;
    eventName: string;
}

/**
 * Interface to output telemetry events.
 * Implemented by hosting app / loader
 * @public
 */
export declare interface ITelemetryBaseLogger {
    send(event: ITelemetryBaseEvent, logLevel?: LogLevel): void;
    minLogLevel?: LogLevel;
}

/**
 * JSON-serializable properties, which will be logged with telemetry.
 * @public
 */
export declare interface ITelemetryBaseProperties {
    /**
     * Properties of a telemetry event. They are string-indexed, and their values restricted to a known set of
     * types (optionally "wrapped" with {@link Tagged}).
     */
    [index: string]: TelemetryBaseEventPropertyType | Tagged<TelemetryBaseEventPropertyType>;
}

/**
 * Warning emitted when requests to storage are being throttled
 * @alpha
 */
export declare interface IThrottlingWarning extends IErrorBase {
    /**
     * {@inheritDoc IErrorBase.errorType}
     */
    readonly errorType: typeof FluidErrorTypes.throttlingError;
    readonly retryAfterSeconds: number;
}

/* Excluded from this release type: IUsageError */

/**
 * Specify levels of the logs.
 * @public
 */
export declare const LogLevel: {
    readonly verbose: 10;
    readonly default: 20;
    readonly error: 30;
};

/**
 * Specify a level to the log to filter out logs based on the level.
 * @public
 */
export declare type LogLevel = (typeof LogLevel)[keyof typeof LogLevel];

/**
 * Does the type replacement by changing types of {@link IEventThisPlaceHolder} to `TThis`
 * @public
 */
export declare type ReplaceIEventThisPlaceHolder<L extends any[], TThis> = L extends any[] ? {
    [K in keyof L]: L[K] extends IEventThisPlaceHolder ? TThis : L[K];
} : L;

/**
 * A property to be logged to telemetry may require a tag indicating the value may contain sensitive data.
 * This type wraps a value of the given type V in an object along with a string tag (type can be further specified as T).
 *
 * This indicates that the value should be organized or handled differently by loggers in various first or third
 * party scenarios. For example, tags are used to mark data that should not be stored in logs for privacy reasons.
 * @public
 */
export declare interface Tagged<V, T extends string = string> {
    value: V;
    tag: T;
}

/**
 * Property types that can be logged.
 *
 * @remarks Logging entire objects is considered extremely dangerous from a telemetry point of view because people can
 * easily add fields to objects that shouldn't be logged and not realize it's going to be logged.
 * General best practice is to explicitly log the fields you care about from objects.
 * @public
 */
export declare type TelemetryBaseEventPropertyType = string | number | boolean | undefined;

/**
 * Transforms the event overload by replacing {@link IEventThisPlaceHolder} with `TThis` in the event listener
 * arguments and having the overload return `TTHis` as well
 * @public
 */
export declare type TransformedEvent<TThis, E, A extends any[]> = (event: E, listener: (...args: ReplaceIEventThisPlaceHolder<A, TThis>) => void) => TThis;

export { }
